/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2007 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Application
    g3dFoam

Description
    Density-based compressible flow solver.

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{

#   include "setRootCase.H"
#   include "createTime.H"
#   include "createMesh.H"
#   include "readThermodynamicProperties.H"
#   include "readG3Dcoefficients.H"
#   include "createFields.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
  
#   include "findNeighbours.H"

  // Print info about boundaries
  forAll(mesh.boundaryMesh(), ibound)
    {
      Info<< "Boundary mesh [" << ibound << "]:" << 
	mesh.boundaryMesh()[ibound] << endl;
    }

  forAll(U.boundaryField(), ibound)
    {
      Info<< "Boundary field U[" << ibound << "]:" << 
	U.boundaryField()[ibound] << endl;
    }

  forAll(p.boundaryField(), ibound)
    {
      Info<< "Boundary field p[" << ibound << "]:" << 
	p.boundaryField()[ibound] << endl;
    }
 
  forAll(T.boundaryField(), ibound)
    {
      Info<< "Boundary field T[" << ibound << "]:" << 
	T.boundaryField()[ibound] << endl;
    }
 


  // Dummy to set correct dimension
  dimensionedScalar zeroPerTime  = 
    dimensionedScalar("zeroPerTime",dimless/dimTime,0.0);
  
  // Flux of rho
  volScalarField Drho
    (
     IOobject
     (
      "Drho",
      runTime.timeName(),
      mesh,
      IOobject::NO_READ,
      IOobject::NO_WRITE
      ),
     rho*zeroPerTime
     );
  
  // Flux of rhoU
  volVectorField DrhoU
    (
     IOobject
     (
      "DrhoU",
      runTime.timeName(),
      mesh,
      IOobject::NO_READ,
      IOobject::NO_WRITE
      ),
     rhoU*zeroPerTime
     );
  
  // Flux of rhoE
  volScalarField DrhoE
    (
     IOobject
     (
      "DrhoE",
      runTime.timeName(),
      mesh,
      IOobject::NO_READ,
      IOobject::NO_WRITE
      ),
     rhoE*zeroPerTime
     );
  
  // rho0
  volScalarField rho0
    (
     IOobject
     (
      "rho0",
      runTime.timeName(),
      mesh,
      IOobject::NO_READ,
      IOobject::NO_WRITE
      ),
     rho
     );
  
  // rhoU0
  volVectorField rhoU0
    (
     IOobject
     (
      "rhoU0",
      runTime.timeName(),
      mesh,
      IOobject::NO_READ,
      IOobject::NO_WRITE
      ),
     rhoU
     );
  
  // rhoE0
  volScalarField rhoE0
    (
     IOobject
     (
      "rhoE0",
      runTime.timeName(),
      mesh,
      IOobject::NO_READ,
      IOobject::NO_WRITE
      ),
     rhoE
     );
  

  Info<< "\nStarting time loop\n" << endl;
  for (runTime++; !runTime.end(); runTime++)
    {
      Info<< "Time = " << runTime.timeName() << endl;
      
      // Calculate CFL#
      scalar cfl = max(mesh.deltaCoeffs()*(mag(U)+c))*
	runTime.deltaT().value();
      Info<< "CFLmax=" << cfl << endl;

      /////////////////////////////////
      // Do three stage Runge-Kutta  //
      /////////////////////////////////
      
      rho0  = rho;
      rhoU0 = rhoU;
      rhoE0 = rhoE;

      // Calculate fluxes over faces
#     include "flux.H"

      // Uptade rho,rhoU,rhoE
      rho  = rho0  + runTime.deltaT()*Drho ;
      rhoU = rhoU0 + runTime.deltaT()*DrhoU;
      rhoE = rhoE0 + runTime.deltaT()*DrhoE;

      // Calculate rho0,rhoU0,rhoE0
      rho0  = 0.5*(rho0  + rho);
      rhoU0 = 0.5*(rhoU0 + rhoU);
      rhoE0 = 0.5*(rhoE0 + rhoE);

      // time = time + deltaT;

      // Auxvr / primitive variables
      U = rhoU/rho;
      p = (gamma-1)*(rhoE - 0.5*rho*magSqr(U));
      T = p/(R*rho);
      c = sqrt(gamma*R*T);
      
      // Calculate fluxes over faces
#     include "flux.H"
     
      // Update rho,rhoU,rhoE
      rho  = rho0  + 0.5*runTime.deltaT()*Drho ;
      rhoU = rhoU0 + 0.5*runTime.deltaT()*DrhoU;
      rhoE = rhoE0 + 0.5*runTime.deltaT()*DrhoE;

      // Auxvr / primitive variables
      U = rhoU/rho;
      p = (gamma-1)*(rhoE - 0.5*rho*magSqr(U));
      T = p/(R*rho);
      c = sqrt(gamma*R*T);
      
      // Calculate fluxes over faces
#     include "flux.H"
     
      // Update rho,rhoU,rhoE
      rho  = rho0  + 0.5*runTime.deltaT()*Drho ;
      rhoU = rhoU0 + 0.5*runTime.deltaT()*DrhoU;
      rhoE = rhoE0 + 0.5*runTime.deltaT()*DrhoE;
      
      // Auxvr / primitive variables
      U = rhoU/rho;
      p = (gamma-1)*(rhoE - 0.5*rho*magSqr(U));
      T = p/(R*rho);
      c = sqrt(gamma*R*T);

      // Runge-Kutta end


      // Save solution
      runTime.write();

      // Maximum change in density
      dimensionedScalar rhoMax = max(mag(rho - rho.oldTime()));
      Info<< "Rmax=" << rhoMax.value() << endl;

      Info<< "ExecutionTime = " << runTime.elapsedCpuTime() << " s"
	  << "  ClockTime = " << runTime.elapsedClockTime() << " s"
	  << nl << endl;
    }

  Info<< "End\n" << endl;
  
  return(0);
}


// ************************************************************************* //
